package nc.gl.pub.print.utils;

import java.awt.Graphics2D;
import java.io.File;
import java.io.FileOutputStream;

import nc.ui.pub.print.IDataSource;
import nc.ui.pub.print.output.DefaultDataSourceOrganizer;
import nc.ui.pub.print.output.DefaultTemplateDataAccessor;
import nc.ui.pub.print.page.DocGroupUtil;
import nc.ui.pub.print.version55.print.merge.CopyConfig;
import nc.ui.pub.print.version55.print.merge.MergedPage;
import nc.ui.pub.print.version55.print.merge.MergedResult;
import nc.ui.pub.print.version55.print.output.OutputDocument;
import nc.ui.pub.print.version55.print.output.OutputDocumentUtil;
import nc.ui.pub.print.version55.print.output.paint.OutputContext;
import nc.ui.pub.print.version55.print.output.paint.PageFactory;
import nc.ui.pub.print.version55.print.pagesplit.core.atomsplit.SplitContext;
import nc.ui.pub.print.version55.print.template.Paper;
import nc.ui.pub.print.version55.print.template.Template;
import nc.ui.pub.print.version55.print.template.TemplateAdapter;
import nc.vo.pub.print.MergeConfig;

import com.lowagie.text.Document;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfWriter;

public class GlConvertPdfUtils {

	/**
	 * 总账报表导出PDF
	 *
	 * @param pk_group
	 *            集团
	 * @param pk_user
	 *            用户
	 * @param nodeKey
	 *            打印模板标识
	 * @param filePath
	 *            //保存文件路径
	 * @return 文件暂存路径
	 * @throws Exception
	 */
	public static String pdfExport(String pk_group, String pk_user,
			String nodeCode, String nodeKey, String filePath,
			IDataSource... singleObj) throws Exception {
		DefaultTemplateDataAccessor m_printDataAccessor = new DefaultTemplateDataAccessor();
		DefaultDataSourceOrganizer m_dsOrganizer = new DefaultDataSourceOrganizer();
		for (IDataSource iDataSource : singleObj) {
			m_dsOrganizer.addDataSource(iDataSource);
		}
		m_printDataAccessor.setTemplateID(pk_group, null, nodeCode, pk_user,
				null, nodeKey, null);
		Template t = TemplateAdapter
				.translate((DefaultTemplateDataAccessor) m_printDataAccessor);
		OutputDocument docs = DocGroupUtil.orgOutputDocument(
				(DefaultTemplateDataAccessor) m_printDataAccessor,
				m_dsOrganizer);
		Paper pageSize = null;
		// Paper pageSize =
		// PrintCache.getInstance().loadPaper(PrintCache.getTemplateKey(docs));
		if (pageSize == null)
			pageSize = t.getOutPageSize();
		MergeConfig mc = null;
		// MergeConfig mc =
		// PrintCache.getInstance().loadMergeConfig(PrintCache.getTemplateKey(docs));
		if (mc == null)
			mc = TemplateAdapter
					.translateMergeConfig((DefaultTemplateDataAccessor) m_printDataAccessor);

		CopyConfig cc = TemplateAdapter
				.translateCopyConfig((DefaultTemplateDataAccessor) m_printDataAccessor);

		OutputContext context = new OutputContext();
		// 格式化导出数据
		OutputDocumentUtil.formatDs(docs);
		PageFactory pages = new PageFactory(docs);
		// pages.setPageSize(paper);

		pages.setPageSize(pageSize);
		pages.setMergeConfig(mc);
		pages.setCopyConfig(cc);
		pages.getSplitContext().setOperation(SplitContext.OPERATION_PDF);
		// 分页上下文
		context.setSplitContext(pages.getSplitContext());
		int lenght = 1;

		pages.doSplit();

		int toPage = pages.getResult().getPages().size();
		if (lenght > 10) {
			toPage = toPage + (lenght - 1) / 10;
		}
		createFile(filePath, 0, toPage, context, pages.getResult(),
				pages.getPageSize());

		return filePath;
	}
	
	/**
	 * 总账报表导出PDF
	 * @param templateId 模板id
	 * @param filePath 存储路径
	 * @param singleObj 数据源
	 * @return
	 * @throws Exception
	 */
	public static String pdfExport(String templateId, String filePath,
			IDataSource... singleObj) throws Exception {
		DefaultTemplateDataAccessor m_printDataAccessor = new DefaultTemplateDataAccessor();
		DefaultDataSourceOrganizer m_dsOrganizer = new DefaultDataSourceOrganizer();
		for (IDataSource iDataSource : singleObj) {
			m_dsOrganizer.addDataSource(iDataSource);
		}
		m_printDataAccessor.setTemplateID(templateId,false);
		Template t = TemplateAdapter
				.translate((DefaultTemplateDataAccessor) m_printDataAccessor);
		OutputDocument docs = DocGroupUtil.orgOutputDocument(
				(DefaultTemplateDataAccessor) m_printDataAccessor,
				m_dsOrganizer);
		Paper pageSize = null;
		// Paper pageSize =
		// PrintCache.getInstance().loadPaper(PrintCache.getTemplateKey(docs));
		if (pageSize == null)
			pageSize = t.getOutPageSize();
		MergeConfig mc = null;
		// MergeConfig mc =
		// PrintCache.getInstance().loadMergeConfig(PrintCache.getTemplateKey(docs));
		if (mc == null)
			mc = TemplateAdapter
					.translateMergeConfig((DefaultTemplateDataAccessor) m_printDataAccessor);

		CopyConfig cc = TemplateAdapter
				.translateCopyConfig((DefaultTemplateDataAccessor) m_printDataAccessor);

		OutputContext context = new OutputContext();
		// 格式化导出数据
		OutputDocumentUtil.formatDs(docs);
		PageFactory pages = new PageFactory(docs);
		// pages.setPageSize(paper);

		pages.setPageSize(pageSize);
		pages.setMergeConfig(mc);
		pages.setCopyConfig(cc);
		pages.getSplitContext().setOperation(SplitContext.OPERATION_PDF);
		// 分页上下文
		context.setSplitContext(pages.getSplitContext());
		int lenght = 1;

		pages.doSplit();

		int toPage = pages.getResult().getPages().size();
		if (lenght > 10) {
			toPage = toPage + (lenght - 1) / 10;
		}
		createFile(filePath, 0, toPage, context, pages.getResult(),
				pages.getPageSize());

		return filePath;
	}

	/**
	 * 保存PDF
	 * 
	 * @param path
	 * @param fromPage
	 * @param toPage
	 * @param context
	 * @param pages
	 * @param paper
	 * @throws Exception
	 */
	private static void createFile(String path, int fromPage, int toPage,
			OutputContext context, MergedResult pages, Paper paper)
			throws Exception {
		File file = new File(path);

		File fileParent = file.getParentFile();
		if (!fileParent.exists()) {// 判断路径是否存在
			fileParent.mkdirs();
		}
		if (file.exists()) {// 判断文件是否存在
			file.delete();
			file.createNewFile();
		}
		double pageWidth = paper.getSize().getWidth().getPt();
		double pageHeight = paper.getSize().getHeight().getPt();

		Rectangle rect = new Rectangle((float) pageWidth, (float) pageHeight);
		if (!paper.isDirection())
			rect.rotate();

		Document document = new Document(rect);
		PdfWriter pw = null;
		pw = PdfWriter.getInstance(document, new FileOutputStream(path));

		document.open();
		// ProgressMonitor monitor = new ProgressMonitor();
		PdfContentByte cb = pw.getDirectContent();
		Graphics2D g2 = cb.createGraphicsShapes(rect.getWidth(),
				rect.getHeight());

		// monitor.setTotal(1);
		MergedPage page = null;
		for (int i = fromPage; i < toPage; i++) {
			page = pages.getPages().get(i);

			context.setCurMergePage(page);
			page.paint(g2, context);

			// monitor.setValue(monitor.getValue() + 1);

			if (i != toPage)
				document.newPage();
		}

		document.close();
	}
}
